﻿
using HarmonyLib;
using TenCrowns.AppCore;
using TenCrowns.GameCore;
using Mohawk.SystemCore;
using System;

namespace ReligionVictory
{
    public class ReligionVictoryClass : ModEntryPointAdapter
    {
        public const string MY_HARMONY_ID = "dale.ReligionVictory.patch";
        public Harmony harmony;

        public override void Initialize(ModSettings modSettings)
        {
            harmony = new Harmony(MY_HARMONY_ID);
            harmony.PatchAll();
        }

        public override void Shutdown()
        {
            harmony.UnpatchAll(MY_HARMONY_ID);
        }
    }

    [HarmonyPatch(typeof(Unit), nameof(Unit.canPurgeReligion), new Type[] { typeof(Tile), typeof(ReligionType), typeof(bool) })]

    public class PatchReligionVictoryClassPurgeReligion
    {
        static bool Prefix(ref bool __result, Unit __instance, Tile pTile, ReligionType eReligion, bool bTestEnabled = true)
        {
            if (__instance.info().meSpreadReligion == ReligionType.NONE)
            {
                __result = false;
                return false;
            }

            if (eReligion == __instance.info().meSpreadReligion)
            {
                __result = false;
                return false;
            }

            if (eReligion == __instance.player().getStateReligion())
            {
                __result = false;
                return false;
            }

            if (__instance.game().isPaganReligion(eReligion))
            {
                __result = false;
                return false;
            }

            City pCity = pTile.cityTerritory();

            if (pCity == null)
            {
                __result = false;
                return false;
            }

            //                if (pCity.getPlayer() != __instance.getPlayer())
            //                {
            //                    __result = false;
            //                }

            if (!(pCity.canRemoveReligion(eReligion)))
            {
                __result = false;
                return false;
            }

            if (!(__instance.player().isPurgeReligionsUnlock()))
            {
                __result = false;
                return false;
            }

            if (bTestEnabled)
            {
                if (!__instance.canUseUnit(__instance.player(), bPlayerOnly: true))
                {
                    __result = false;
                    return false;
                }

                if (!__instance.canAct(__instance.player(), __instance.game().infos().Globals.UNIT_PURGE_COST))
                {
                    return false;
                }
            }
            __result = true;
            return false;
        }

        [HarmonyPatch(typeof(Tile), nameof(Tile.canEnterTerritory), new Type[] { typeof(UnitType), typeof(PlayerType), typeof(TribeType), typeof(PlayerType), typeof(TribeType) })]

        public class PatchReligionVictoryClassEnterTerritory
        {
            static bool Prefix(ref bool __result, Tile __instance, UnitType eUnit, PlayerType ePlayerUnit, TribeType eTribeUnit, PlayerType ePlayerTile, TribeType eTribeTile)
            {
                if ((ePlayerTile == PlayerType.NONE) && (eTribeTile == TribeType.NONE))
                {
                    __result = true;
                    return false;
                }

                if (__instance.infos().unit(eUnit).mbReligious)
                {
                    __result = true;
                    return false;
                }

                if (ePlayerUnit != PlayerType.NONE)
                {
                    PlayerType eCapturePlayer = __instance.getVisibleCapturePlayer(__instance.game().player(ePlayerUnit).getTeam());

                    if (eCapturePlayer != PlayerType.NONE)
                    {
                        ePlayerTile = eCapturePlayer;
                        eTribeTile = TribeType.NONE;
                    }
                }

                TeamType eTeamUnit = ((ePlayerUnit != PlayerType.NONE) ? __instance.game().player(ePlayerUnit).getTeam() : TeamType.NONE);
                TeamType eTeamTile = ((ePlayerTile != PlayerType.NONE) ? __instance.game().player(ePlayerTile).getTeam() : TeamType.NONE);

                if (eTeamUnit == eTeamTile)
                {
                    __result = true;
                    return false;
                }

                if (__instance.game().isHostile(eTeamUnit, eTribeUnit, eTeamTile, eTribeTile))
                {
                    __result = true;
                    return false;
                }

                if (ePlayerUnit != PlayerType.NONE)
                {
                    if (__instance.game().player(ePlayerUnit).isHideUnitUnlock(eUnit))
                    {
                        __result = true;
                        return false;
                    }
                }

                InfoDiplomacy diplomacy = __instance.infos().diplomacy(__instance.game().getDiplomacy(eTeamUnit, eTribeUnit, eTeamTile, eTribeTile));

                __result = ((__instance.infos().unit(eUnit).mbCaravan) ? !diplomacy.mbHostile : diplomacy.mbPeace);
                return false;
            }
        }

        [HarmonyPatch(typeof(Game), nameof(Game.checkVictory), new Type[] { typeof(VictoryType), typeof(TeamType), typeof(int) })]

        public class PatchReligionVictoryClassCheckVictory        {
            static bool Prefix(ref bool __result, Game __instance, VictoryType eVictory, TeamType eTeam, int iMarginPercent = 0)
            {
                if (!__instance.isVictoryEnabled(eVictory))
                {
                    __result = false;
                    return false;
                }

                if (!__instance.isTeamAlive(eTeam))
                {
                    __result = false;
                    return false;
                }

                InfoVictory victory = __instance.infos().victory(eVictory);

                int iOwnVPs = __instance.countTeamVPs(eTeam);

                iOwnVPs = __instance.infos().utils().modify(iOwnVPs, iMarginPercent);

                if (victory.mbConquest)
                {
                    if (__instance.getNumTeamsInt() == 1)
                    {
                        __result = false;
                        return false;
                    }
                    for (TeamType eLoopTeam = 0; eLoopTeam < __instance.getNumTeams(); eLoopTeam++)
                    {
                        if (eTeam != eLoopTeam)
                        {
                            if (__instance.isTeamAlive(eLoopTeam))
                            {
                                if (!__instance.isTeamSurrendered(eLoopTeam))
                                {
                                    if (__instance.getTeamAlliance(eLoopTeam) != eTeam)
                                    {
                                        __result = false;
                                        return false;
                                    }
                                    else if (__instance.isSinglePlayer())
                                    {
                                        __result = false;
                                        return false;
                                    }
                                    else if (__instance.countTeamVPs(eLoopTeam) > iOwnVPs)
                                    {
                                        __result = false;
                                        return false;
                                    }
                                }
                            }
                        }
                    }
                }
                if (victory.mbAmbitions)
                {
                    bool bFound = false;
                    for (PlayerType ePlayer = 0; ePlayer < __instance.getNumPlayers(); ePlayer++)
                    {
                        if (__instance.player(ePlayer).getTeam() == eTeam)
                        {
                            int iNumAmbitions = __instance.player(ePlayer).countAmbitionsFinished();

                            iNumAmbitions = __instance.infos().utils().modify(iNumAmbitions, iMarginPercent);

                            if (iNumAmbitions >= __instance.infos().Globals.MAX_AMBITIONS)
                            {
                                bFound = true;
                                break;
                            }
                        }
                    }
                    if (!bFound)
                    {
                        __result = false;
                        return false;
                    }
                }
                if (victory.miPercentVP != 0)
                {
                    if (100 * iOwnVPs < __instance.getVPToWin() * victory.miPercentVP)
                    {
                        __result = false;
                        return false;
                    }
                }
                if (victory.miOpponentMaxPointPercent != 0)
                {
                    if (__instance.getNumTeamsInt() == 1)
                    {
                        __result = false;
                        return false;
                    }
                    if (!__instance.isTeamHuman(eTeam))
                    {
                        __result = false;
                        return false;
                    }
                    for (TeamType eLoopTeam = 0; eLoopTeam < __instance.getNumTeams(); eLoopTeam++)
                    {
                        if (__instance.isTeamAlive(eLoopTeam))
                        {
                            if (eTeam != eLoopTeam)
                            {
                                if (__instance.countTeamVPs(eLoopTeam) * 100 > iOwnVPs * victory.miOpponentMaxPointPercent)
                                {
                                    __result = false;
                                    return false;
                                }
                            }
                        }
                    }
                }
                if (__instance.HelpText.TEXT(victory.meName) == "Religious Victory")
                {
                    for (PlayerType ePlayer = 0; ePlayer < __instance.getNumPlayers(); ePlayer++)
                    {
                        if (__instance.player(ePlayer).getTeam() == eTeam && __instance.player(ePlayer).hasStateReligion() && __instance.player(ePlayer).countHolyCities() > 0)
                        {
                            ReligionType eReligion = __instance.player(ePlayer).getStateReligion();
                            bool bFound = false;
                            foreach (City pCity in __instance.getCities())
                            {
                                if (pCity.isReligionHolyCity(eReligion))
                                {
                                    if (pCity.player() == __instance.player(ePlayer))
                                    {
                                        bFound = true;
                                    }
                                }
                            }
                            if (bFound)
                            {
                                int total = 0;
                                for (PlayerType ePlayer2 = 0; ePlayer2 < __instance.getNumPlayers(); ePlayer2++)
                                {
                                    total = total + __instance.player(ePlayer2).countReligionCities(eReligion);
                                }
                                if ((__instance.getNumCities() * 0.75) <= total)
                                {
                                    __result = true;
                                    return false;
                                }
                            }
                        }
                    }
                    __result = false;
                    return false;
                }
                if (victory.miMinTurns != 0)
                {
                    if (__instance.getTurn() < victory.miMinTurns)
                    {
                        __result = false;
                        return false;
                    }
                }
                __result = true;
                return false;
            }
        }
    }
}
